"""
Pypodo scripts
"""

import re
from columnar import columnar
from pypodo.properties import (
    REGEX_INDEX,
)
from pypodo.config import (
    todofilefromconfig,
    read_config_boolean
)
from pypodo.args import compute_args
from pypodo.print import (
    printwarning,
    printlinetodo,
)



def find(openfile=open):
    """
    Search with regex in the todofile
    """
    empty = True
    headers = ["index","task","tags"]
    data = []
    with open(todofilefromconfig(), "r") as todofile:
        for line in todofile.readlines():
            search = compute_args().search
            if re.findall(search, line.rstrip("\n")):
                data.append(printlinetodo(line))
                empty = False
    if empty:
        printwarning("the filtered todolist is empty")
    else:
        if compute_args().condensate:
            table = columnar(data, no_borders=True, wrap_max=0)
        else:
            if (read_config_boolean("FONCTIONAL", "condensate", "False")== "True"):
                table = columnar(data, no_borders=True, wrap_max=0)
            else:    
                table = columnar(data, headers, no_borders=False, wrap_max=0)            
        print(table)    


"""
Pypodo scripts
"""


import re


from pypodo.properties import (
    REGEX_INDEX,
    REGEX_SPACE_OR_ENDLINE,
)
from pypodo.config import (
    todofilefromconfig,

)
from pypodo.args import compute_args
from pypodo.print import (
    printwarning,
    printinfo,
    printerror,
)
from pypodo.check import check



def untag(openfile=open):
    """
    Untag tasks from the todofile
    """
    if len(compute_args().untag) >= 2:
        tagtodel = compute_args().untag[0]
        if not re.findall("^[^ #]+$", tagtodel):
            printerror("the tag has not a valid format - " + tagtodel)
        else:
            # loop on the indexes
            for increment in range(1, len(compute_args().untag)):
                index = compute_args().untag[increment]
                # check the numeric format of the index
                if re.findall(REGEX_INDEX, index):
                    untagone(index, tagtodel, openfile)
                elif re.findall("^\\d+-\\d+$", index) and int(
                    index.split("-")[0]
                ) < int(index.split("-")[1]):
                    for indexlist in range(
                        int(index.split("-")[0]),
                        int(index.split("-")[1]) + 1,
                    ):
                        untagone(str(indexlist), tagtodel, openfile)
                else:
                    printwarning(
                        "the index to untag is not in numeric format - "
                        + index
                    )

    else:
        printerror(
            "2 or more parameters is needed for pypodo untag : the tag to delete and"
            " the indexes of the task whose tags to delete"
        )


def untagone(index, tagtodel, openfile):
    """
    Untag on task
    """
    index_trouve = False
    with openfile(todofilefromconfig(), "r") as todofile:
        lines = todofile.readlines()
    with openfile(todofilefromconfig(), "w") as todofile:
        for line in lines:
            if not re.findall("^" + index + " ", line):
                todofile.write(line)
            else:
                if re.findall(
                    " #"
                    + re.escape(tagtodel)
                    + REGEX_SPACE_OR_ENDLINE,
                    line.rstrip("\n"),
                ):
                    newline = re.sub(
                        "#"
                        + re.escape(tagtodel)
                        + REGEX_SPACE_OR_ENDLINE,
                        "",
                        line,
                    ).rstrip("\n")
                    todofile.write(newline.rstrip() + "\n")
                    printinfo(
                        "tag deleted from the task of the todolist - "
                        + line.rstrip("\n")
                        + " -> "
                        + newline.rstrip()
                    )
                else:
                    todofile.write(line)
                    printwarning(
                        "no tags is deleted from the todolist for the task - "
                        + line.rstrip("\n")
                    )
                index_trouve = True
    if not index_trouve:
        printwarning("no task with index - " + index)



"""
pypodo argparse gestion
"""

import argparse


class CustomHelpFormatter(argparse.HelpFormatter):
    def _format_action_invocation(self, action):
        if not action.option_strings or action.nargs == 0:
            return super()._format_action_invocation(action)
        default = self._get_default_metavar_for_optional(action)
        args_string = self._format_args(action, default)
        return ", ".join(action.option_strings) + " " + args_string

    def _format_args(self, action, default_metavar):
        get_metavar = self._metavar_formatter(action, default_metavar)
        if action.nargs == argparse.ONE_OR_MORE:
            return "%s" % get_metavar(1)
        else:
            return super(CustomHelpFormatter, self)._format_args(
                action, default_metavar
            )


def compute_args():
    """
    check args and return them
    """
    my_parser = argparse.ArgumentParser(
        description="pypodo is a todolist tool which works in your terminal. It has a mecanism of indexes and tags.",
        epilog="""
        Full documentation at: <https://github.com/thib1984/pypodo>.
        Report bugs to <https://github.com/thib1984/pypodo/issues>.
        MIT Licence.
        Copyright (c) 2021 thib1984.
        This is free software: you are free to change and redistribute it.
        There is NO WARRANTY, to the extent permitted by law.
        Written by thib1984.""",
        formatter_class=CustomHelpFormatter,
    )
    my_group = my_parser.add_mutually_exclusive_group()
    my_group.add_argument(
        "-a",
        "--add",
        action="store",
        nargs="+",
        type=str,
        metavar="ITEM ...",
        help="add ITEM(s) in the todolist",
    )
    my_group.add_argument(
        "-w",
        "--warning",
        action="store_true",
        help="print the todolist filtered with TAG(s) alert and warning",
    )    
    my_group.add_argument(
        "-d",
        "--delete",
        action="store",
        metavar="INDEX ...",
        nargs="+",
        type=str,
        help="delete item with the given INDEX(es)",

    )
    my_group.add_argument(
        "--tag",
        action="store",
        nargs="+",
        type=str,
        metavar="TAG INDEX ...",
        help="TAG item with the given INDEX(es)",
    )
    my_group.add_argument(
        "--untag",
        action="store",
        nargs="+",
        type=str,
        metavar="TAG INDEX ...",
        help="unTAG item with the given INDEX(es)",
    )
    my_parser.add_argument(
        "-f",
        "--filter",
        action="store",
        nargs="+",
        type=str,
        metavar="TAG ...",
        help="print the todolist filtered with TAG(s) given",
    )
    my_parser.add_argument(
        "-e",
        "--exclude",
        action="store",
        nargs="+",
        type=str,
        metavar="TAG ...",
        help="print the todolist with TAG(s) given excluden",
    )
    my_group.add_argument(
        "-o",
        "--order",
        action="store_true",
        help="order the todolist",
    )
    my_group.add_argument(
        "-b",
        "--backup",
        action="store_true",
        help="backup the todolist",
    )
    my_group.add_argument(
        "-s",
        "--search",
        action="store",
        type=str,
        metavar="REGEX",
        help="search in the todolist with the REGEX given",
    )

    my_group.add_argument(
        "--info",
        action="store_true",
        help="print the infos",
    )
    my_group.add_argument(
        "-u",
        "--update",
        action="store_true",
        help="update pypodo",
    )

    my_parser.add_argument(
        "-n",
        "--nocolor",
        action="store_true",
        help="disable color in sysout",
    )

    my_parser.add_argument(
        "-c",
        "--condensate",
        action="store_true",
        help="condensate sysout",
    )

    my_parser.add_argument(
        "-D",
        "--day",
        action="store_true",
        help="add tag actual day",
    )

    my_parser.add_argument(
        "-W",
        "--week",
        action="store_true",
        help="add tag next week",
    )

    my_parser.add_argument(
        "-M",
        "--month",
        action="store_true",
        help="add tag next month",
    )

    my_parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="verbose mode",
    )
    
    args = my_parser.parse_args()
    return args



"""
Pypodo scripts
"""

import os
import sys
import time
from shutil import copyfile

from pypodo.config import (
    todofilefromconfig,
    todobackupfolderfromconfig,
)
from pypodo.print import (
    printinfo,
    printerror,
)


def backup(openfile=open):
    """
    Backup the todofile
    """
    dir_exists = os.path.exists(todobackupfolderfromconfig())
    if not dir_exists:
        try:
            os.makedirs(todobackupfolderfromconfig())
        except PermissionError:
            printerror(
                "permission error to create the backup folder : "
                + todobackupfolderfromconfig()
            )
            sys.exit()
        printinfo("creating todolist backup folder")
    time_suffix = time.strftime("%Y%m%d%H%M%S")
    todo_backup_name = "todo" + time_suffix
    backup_name = os.path.join(todobackupfolderfromconfig(),todo_backup_name)
    try:
        copyfile(todofilefromconfig(), backup_name)
    except PermissionError:
        printerror(
            "permission error to create the backup folder : "
            + todobackupfolderfromconfig()
        )
        sys.exit()
    printinfo("creating todolist backup - " + backup_name)



import pkg_resources
import os
from pypodo.properties import TODO_RC_FILE
from pypodo.config import get_user_config_directory_pypodo, todofilefromconfig, todobackupfolderfromconfig

def version():
    """
    entry point for --version
    """
    print("version pypodo            : "
        + pkg_resources.get_distribution("pypodo").version
    )
    print("location of todo file     : " + todofilefromconfig())
    print("location of config file   : " + os.path.join(get_user_config_directory_pypodo(), TODO_RC_FILE))
    print("location of backup folder : " + todobackupfolderfromconfig())


"""
Pypodo scripts
"""

import re
from pypodo.config import (
    read_config_date_format,
    todofilefromconfig,
)
from dateutil.relativedelta import relativedelta
from pypodo.args import compute_args
from pypodo.print import (
    printwarning,
    printinfo,
)
from datetime import datetime, timedelta

def add(openfile=open):
    """
    Add a task to the todofile
    """
    with openfile(todofilefromconfig(), "r") as todofile:
        lines = todofile.readlines()
        # index calculation
        if len(lines) > 0:
            last_line = lines[len(lines) - 1]
            index = int(last_line.split()[0]) + 1
        else:
            index = 1
    with openfile(todofilefromconfig(), "a") as todofile:
        # loop on the indexes
        for task in compute_args().add:
            if compute_args().day:    
                task = task + " #"+datetime.now().strftime(read_config_date_format("SYSTEM", "formatdate", "%Y%m%d"))
            if compute_args().week:
                task = task + " #"+(datetime.now()+relativedelta(weeks=1)).strftime(read_config_date_format("SYSTEM", "formatdate", "%Y%m%d"))
            if compute_args().month:
                task = task + " #"+(datetime.now()+relativedelta(months=1)).strftime(read_config_date_format("SYSTEM", "formatdate", "%Y%m%d"))                              
            # check format : words* #tag1 #tag2 : task at free format,
            # tags in one word prefixed by #
            if not re.findall("^([^#]|([^ ]#))*( #[^ #]+)*$", task):
                printwarning(
                    "the task has not a valid format - " + task
                )
            else:
                # adding task to the todolist
                todofile.write(str(index) + " " + task + "\n")
                printinfo(
                    "task is added to the todolist - "
                    + str(index)
                    + " "
                    + task
                )
                index = index + 1



"""
Pypodo scripts
"""

import argparse
import os
import re
import sys
import time
import base64
import hashlib
from pathlib import Path
from shutil import copyfile
from datetime import datetime
from datetime import date
from pypodo.properties import (
    REGEX_INDEX,
    REGEX_SPACE_OR_ENDLINE,
)
from pypodo.config import (
    read_config,
    read_config_boolean,
    read_config_color,
    read_config_date_format,
    read_config_int,
    read_config_level,
    todofilefromconfig,
    todobackupfolderfromconfig,
)
from termcolor import colored
from pypodo.args import compute_args
from pypodo.update import update
from pypodo.version import version
from pypodo.print import (
    printdebug,
    printwarning,
    printinfo,
    printlinetodo,
    printerror,
)
from pypodo.check import check
from pypodo.list import listtask
from pypodo.add import add
from pypodo.delete import delete
from pypodo.sort import sort
from pypodo.untag import untag
from pypodo.tag import tag
from pypodo.backup import backup
from pypodo.search import find

# entrypoint
def pypodo(openfile=open):
    """
    Entrypoint
    """
    compute_args()
    if check(openfile):
        if compute_args().filter:
            listtask(openfile)
        elif compute_args().add:
            add(openfile)
        elif compute_args().delete:
            delete(openfile)
        elif compute_args().order:
            sort(openfile)
            listtask(openfile)
        elif compute_args().untag:
            untag(openfile)
        elif compute_args().tag:
            tag(openfile)
        elif compute_args().backup:
            backup(openfile)
        elif compute_args().search:
            find(openfile)
        elif compute_args().info:
            version()
        elif compute_args().update:
            update()
        else:
            listtask(openfile)






"""
Pypodo scripts
"""


import re


from pypodo.properties import (
    REGEX_INDEX,
)
from pypodo.config import (
    todofilefromconfig,
)

from pypodo.print import (
    printwarning,
    printinfo,
)


def sort(openfile=open):
    """
    Reorder the todofile with consecutives indexes
    """
    empty = True
    index = 1
    with openfile(todofilefromconfig(), "r") as todofile:
        lines = todofile.readlines()
    with openfile(todofilefromconfig(), "w") as todofile:
        for line in lines:
            # we replace the existing index by the current index that we increment
            replaced = re.sub("^\\d+ ", str(index) + " ", line)
            index = index + 1
            todofile.write(replaced)
            empty = False
    if empty:
        printwarning("the todolist is empty - nothing to do")
    else:
        printinfo("the todolist is sorted")



from setuptools import setup

setup(
    name="pypodo",
    version="5.2.0",
    description="pypodo is a todolist tool which works in your terminal. It has a mecanism of indexes and tags.",
    long_description="The complete description/installation/use/FAQ is available at : https://github.com/thib1984/pypodo#readme",
    long_description_content_type="text/markdown",
    url="https://github.com/thib1984/pypodo",
    author="thib1984",
    author_email="thibault.garcon@gmail.com",
    license="mit",
    packages=["pypodo"],
    install_requires=["termcolor","columnar","python-dateutil"],
    zip_safe=False,
    entry_points={
        "console_scripts": ["pypodo=pypodo.__init__:pypodo"],
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



"""
Pypodo scripts
"""

import re
from columnar import columnar
from pypodo.properties import (
    REGEX_SPACE_OR_ENDLINE,
)
from pypodo.config import (
    read_config_boolean,
    todofilefromconfig,
)
from pypodo.args import compute_args
from pypodo.print import (
    printwarning,
    printlinetodo,
    listalerttags,
    test_date,
)


def listtask(openfile=open):
    """
    Print the todofile with filters or not
    """
    empty = True
    headers = ["index", "task", "tags"]
    data = []
    with openfile(todofilefromconfig(), "r") as todofile:
        for line in todofile.readlines():
            # without filter -> we print all
            display = True
            if compute_args().filter:
                for tagtofilter in compute_args().filter:
                    # regex to search tags "#toto " or "#toto" at the end of the line
                    if not re.findall(
                        " #"
                        + re.escape(tagtofilter)
                        + REGEX_SPACE_OR_ENDLINE,
                        line.rstrip("\n"),
                    ):
                        display = False
            if compute_args().exclude:
                for tagtoexclude in compute_args().exclude:
                    # regex to search tags "#toto " or "#toto" at the end of the line
                    if re.findall(
                        " #"
                        + re.escape(tagtoexclude)
                        + REGEX_SPACE_OR_ENDLINE,
                        line.rstrip("\n"),
                    ):
                        display = False
            if compute_args().warning:
                isalert = False
                for part in line.split():
                    if (
                        part.startswith("#")
                        and part in listalerttags()
                        or test_date(part[1:]) == "alert"
                        or test_date(part[1:]) == "warning"
                    ):
                        isalert = True
                if isalert == False:
                    display = False
            if display:
                data.append(printlinetodo(line))
                empty = False
    if empty:
        if compute_args().filter:
            printwarning("the filtered todolist is empty")
        else:
            printwarning("the todolist is empty")
    else:
        if compute_args().condensate:
            table = columnar(data, no_borders=True, wrap_max=0)
        else:
            if (
                read_config_boolean(
                    "FONCTIONAL", "condensate", "False"
                )
                == "True"
            ):
                table = columnar(data, no_borders=True, wrap_max=0)
            else:
                table = columnar(
                    data, headers, no_borders=False, wrap_max=0
                )
        print(table)
